WAYPOINT* PATHFINDER::findpath( int sx, int sy, int dx, int dy )
{

	// Start timer (used for statistics display)
	QueryPerformanceCounter( ( LARGE_INTEGER * ) &start_time );





	// Check if start or destination points are blocked
	if ( graph[sx][sy].blocked == 1 || graph[dx][dy].blocked )
	{
		QueryPerformanceCounter( ( LARGE_INTEGER * ) &start_time );
		QueryPerformanceCounter( ( LARGE_INTEGER * ) &end_time );
		return NULL;
	}






	// Reset graph.
	for ( i = 0; i < WORLDWIDTH/ACCURACY; i++ ) 
	{
		for ( j = 0; j < WORLDHEIGHT/ACCURACY; j++ ) 
		{
			graph[i][j].visited = false; 
		} 
	}





	// Reset closedlist.
	for ( i = 0; i < closedlist.size(); i++ )
		cache.push_back( closedlist[i] );
		closedlist.clear();
	// Reset openlist.
	for ( i = 0; i < openlist.size(); i++ )
		cache.push_back( openlist[i] );
		openlist.clear();





	// Initialize first PATHNODE and place it on the openlist
	DEPTH				= 0;
	start				= getnode();
	start->xref			= sx;
	start->yref			= sy;
	start->g			= 0;
	start->h			= GoalDistEstimate( start->xref, start->yref, dx, dy );
	start->f			= start->g + start->h;
	start->parent		= NULL;
	openlist.push_back( start );
	graph[sx][sy].visited = true;





	// Begin
	while ( openlist.empty() == 0 )
	{
		// Extract node with best score (lowest)
		lowestscore = 10000;
		for (  i = 0; i < openlist.size(); i++ )
		{
			if ( openlist[i]->f < lowestscore )
			{
				lowestscore = openlist[i]->f;
				lowestindex = i;
			}
		}
		current = openlist[lowestindex];
		closedlist.push_back(  openlist[ lowestindex ] );
		openlist.erase( &openlist[ lowestindex ] );





		// Check for straight line to target (much quicker)
		if ( controls.F6 == 1 )
		{
			if ( straightline( current->xref, current->yref, dx, dy ) )
				straight = 1;
		}






		// Check if at destination  (or  straight line found)
		if ( (current->xref == dx && current->yref == dy) || straight )
		{	
			index = 0;

			if ( straight )
			{
				straight = 0;
				waypoints[index].x = dx;
				waypoints[index].y = dy;
				index++;
			}

			// Traverse back through visited nodes to reconstruct path
			while ( current != NULL )
			{
				waypoints[index].x = current->xref;
				waypoints[index].y = current->yref;
				current = current->parent;
				index++;
			}
			waypoints[index].x = NULL;


			// Perform filtering routine ?
			if ( controls.F7 == 1 )
				filterwaypoints();


			// Stop timer
			QueryPerformanceCounter( ( LARGE_INTEGER * ) &end_time );

			return NULL;
		}


		// For each direction, test possible path

		// Up right
		if ( current->xref +1 < WORLDWIDTH/ACCURACY && current->yref-1 > 0 &&
			 graph[current->xref+1][current->yref-1].blocked == false &&
			 graph[current->xref+1][current->yref-1].visited == false )
		{
			openlist.push_back( getnode() );
			if ( openlist.back() == NULL )
				return NULL;
			openlist.back()->xref = current->xref+1;
			openlist.back()->yref = current->yref-1;
			openlist.back()->g	  = current->g+4;
			openlist.back()->h	  = GoalDistEstimate( openlist.back()->xref, openlist.back()->yref, dx, dy );
			openlist.back()->f    = openlist.back()->g + openlist.back()->h;
			openlist.back()->parent = current;
			graph[current->xref+1][current->yref-1].visited = true;
		}
		// Down right
		if ( current->xref +1 < WORLDWIDTH/ACCURACY && current->yref+1 < WORLDHEIGHT/ACCURACY &&
			 graph[current->xref+1][current->yref+1].blocked == false &&
			 graph[current->xref+1][current->yref+1].visited == false )
		{
			openlist.push_back( getnode() );
			if ( openlist.back() == NULL )
				return NULL;
			openlist.back()->xref = current->xref+1;
			openlist.back()->yref = current->yref+1;
			openlist.back()->g	  = current->g+4;
			openlist.back()->h	  = GoalDistEstimate( openlist.back()->xref, openlist.back()->yref, dx, dy );
			openlist.back()->f    = openlist.back()->g + openlist.back()->h;
			graph[current->xref+1][current->yref+1].visited = true;
			openlist.back()->parent = current;
		}
		// Down left
		if ( current->xref -1 > 0 && current->yref+1 < WORLDHEIGHT/ACCURACY &&
			 graph[current->xref-1][current->yref+1].blocked == false &&
			 graph[current->xref-1][current->yref+1].visited == false )
		{
			openlist.push_back( getnode() );
			if ( openlist.back() == NULL )
				return NULL;
			openlist.back()->xref = current->xref-1;
			openlist.back()->yref = current->yref+1;
			openlist.back()->g	  = current->g+4;
			openlist.back()->h	  = GoalDistEstimate( openlist.back()->xref, openlist.back()->yref, dx, dy );
			openlist.back()->f    = openlist.back()->g + openlist.back()->h;
			graph[current->xref-1][current->yref+1].visited = true;
			openlist.back()->parent = current;
		}
		// Up left
		if ( current->xref - 1 > 0 && current->yref-1 > 0 &&
			 graph[current->xref-1][current->yref-1].blocked == false &&
			 graph[current->xref-1][current->yref-1].visited == false )
		{
			openlist.push_back( getnode() );
			if ( openlist.back() == NULL )
				return NULL;
			openlist.back()->xref = current->xref-1;
			openlist.back()->yref = current->yref-1;
			openlist.back()->g	  = current->g+4;
			openlist.back()->h	  = GoalDistEstimate( openlist.back()->xref, openlist.back()->yref, dx, dy );
			openlist.back()->f    = openlist.back()->g + openlist.back()->h;
			graph[current->xref-1][current->yref-1].visited = true;
			openlist.back()->parent = current;
		}
		// Up
		if ( current->yref-1 > 0 &&
			 graph[current->xref][current->yref-1].blocked == false &&
			 graph[current->xref][current->yref-1].visited == false )
		{
			openlist.push_back( getnode() );
			if ( openlist.back() == NULL )
				return NULL;
			openlist.back()->xref = current->xref;
			openlist.back()->yref = current->yref-1;
			openlist.back()->g	  = current->g+3;
			openlist.back()->h	  = GoalDistEstimate( openlist.back()->xref, openlist.back()->yref, dx, dy );
			openlist.back()->f    = openlist.back()->g + openlist.back()->h;
			graph[current->xref][current->yref-1].visited = true;
			openlist.back()->parent = current;
		}
		// Right
		if ( current->xref+1 < WORLDWIDTH/ACCURACY &&
			 graph[current->xref+1][current->yref].blocked == false &&
			 graph[current->xref+1][current->yref].visited == false )
		{
			openlist.push_back( getnode() );
			if ( openlist.back() == NULL )
				return NULL;
			openlist.back()->xref = current->xref+1;
			openlist.back()->yref = current->yref;
			openlist.back()->g	  = current->g+3;
			openlist.back()->h	  = GoalDistEstimate( openlist.back()->xref, openlist.back()->yref, dx, dy );
			openlist.back()->f    = openlist.back()->g + openlist.back()->h;
			graph[current->xref+1][current->yref].visited = true;
			openlist.back()->parent = current;
		}
		// Down
		if ( current->yref+1 < WORLDHEIGHT/ACCURACY &&
			 graph[current->xref][current->yref+1].blocked == false &&
			 graph[current->xref][current->yref+1].visited == false )
		{
			openlist.push_back( getnode() );
			if ( openlist.back() == NULL )
				return NULL;
			openlist.back()->xref = current->xref;
			openlist.back()->yref = current->yref+1;
			openlist.back()->g	  = current->g+3;
			openlist.back()->h	  = GoalDistEstimate( openlist.back()->xref, openlist.back()->yref, dx, dy );
			openlist.back()->f    = openlist.back()->g + openlist.back()->h;
			graph[current->xref][current->yref+1].visited = true;
			openlist.back()->parent = current;
		}
		// Left
		if ( current->xref-1 > 0 &&
			 graph[current->xref-1][current->yref].blocked == false &&
			 graph[current->xref-1][current->yref].visited == false )
		{
			openlist.push_back( getnode() );
			if ( openlist.back() == NULL )
				return NULL;
			openlist.back()->xref = current->xref-1;
			openlist.back()->yref = current->yref;
			openlist.back()->g	  = current->g+3;
			openlist.back()->h	  = GoalDistEstimate( openlist.back()->xref, openlist.back()->yref, dx, dy );
			openlist.back()->f    = openlist.back()->g + openlist.back()->h;
			graph[current->xref-1][current->yref].visited = true;
			openlist.back()->parent = current;
		}

	}
	return NULL;
}

/////////////////////////////////////////
inline PATHNODE* PATHFINDER::getnode()
{
	if ( !cache.empty() )
	{
		temp = ( cache.back() );
		cache.pop_back();
		return ( temp );
	}
	else
	{
		return NULL;
	}
}		
/////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////
inline int PATHFINDER::GoalDistEstimate( int sx, int sy, int dx, int dy )
{
	return (( 4 * min(abs(sx-dx),abs(sy-dy)) ) + ( 3 * abs(abs(sx-dx)-abs(sy-dy)) )) * HEURISTIC;
}
////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////
PATHFINDER::PATHFINDER()
{
	// reserve() performs malloc at load time,
	// rather than run time.
	openlist.reserve	( 100000 );
	closedlist.reserve	( 100000 );
	cache.reserve		( 100000 );

	current = NULL;
	start   = NULL;

	straight = 0;

	// Initialise a pool of PATHNODE's
	for ( i = 0; i < 100000; i++ )
	{
		cache.push_back( new PATHNODE );
	}
}
/////////////////////////////////////////


/////////////////////////////////////////
PATHFINDER::~PATHFINDER()
{
	for ( i = 0; i < cache.size(); i++ )
		delete cache[i];
	for ( i = 0; i < openlist.size(); i++ )
		delete openlist[i];
	for ( i = 0; i < closedlist.size(); i++ )
		delete closedlist[i];
}
/////////////////////////////////////////

void PATHFINDER::filterwaypoints()
{
	for ( j = 0; j < 10 && waypoints[2].x != NULL; j++ )
	{
		index = 0;
		for ( i = 0; waypoints[i+2].x != NULL; )
		{
			if ( straightline( waypoints[i].x,   waypoints[i].y,
				               waypoints[i+2].x, waypoints[i+2].y ) )
			{
				buffer[index]	= waypoints[i];
				buffer[++index] = waypoints[i+2];
				i += 2;
			}
			else
			{
				buffer[index]	= waypoints[i];
				buffer[++index] = waypoints[i+1];
				i++;
			}
		}

		buffer[index]     = waypoints[i];
		buffer[index+1]   = waypoints[i+1];
		buffer[index+2].x = NULL;

		// Copy into waypoints
		for ( k = 0; buffer[k].x != NULL; k++ )
			waypoints[k] = buffer[k];

		waypoints[k].x   = NULL;
		waypoints[k+1].x = NULL;
		waypoints[k+2].x = NULL;
	}
}

void PATHFINDER::initialize( CString imagename )
{
	// Load terrain BMP into HDC
	hittest_hdc		= CreateCompatibleDC( NULL ) ;
	hittest_bitmap	= ( HBITMAP ) LoadImage( NULL , imagename, IMAGE_BITMAP , 0 , 0 , LR_LOADFROMFILE ) ;
	SelectObject( hittest_hdc , hittest_bitmap ) ;


	// D3DImagesurface ( used when displaying )
	image = ImageSurfaceFromBMP ( D3DDevice, imagename, D3DCurrentDisplayMode.Format ) ;
	SetRect ( &rect,  0, 0, WORLDWIDTH, WORLDHEIGHT ) ;

	// Create terrain graph
	for ( int x = 0; x < (int)(WORLDWIDTH / ACCURACY); x++ )
	{
		for ( int y = 0; y < (int)(WORLDHEIGHT / ACCURACY); y++ )
		{
			graph[x][y].visited = false; 

			if ( hittest( (int)x*ACCURACY, (int)y*ACCURACY ) )
				graph[x][y].blocked = true; 
			else 
				graph[x][y].blocked = false; 
		}
	}			
}

bool PATHFINDER::hittest( int x, int y )
{
	if ( GetPixel ( hittest_hdc , x , y ) != RGB( 0, 0, 0 ) )
		return 1;
	else
		return 0;
}

int PATHFINDER::round( float i )
{
	// Rounds number to nearest whole integer.
	int temp = (int)i;
	i -= (int)i;

		 if	( i <  0.5f )	return temp;
	else if ( i >= 0.5f )	return temp + 1;
}

bool PATHFINDER::straightline( int sx, int sy, int dx, int dy )
{
	// Function to test for a straight line
	// between two terrain nodes

	float	x = (float)sx;
	float	y = (float)sy;
	float	t = 0;

	float xlength = (float)abs( sx - dx );
	float ylength = (float)abs( sy - dy );
	float toffset = 1 / (xlength + ylength);

	while ( t <= 1 )
	{
		x = ( (1-t) * sx ) + ( t * dx );
		y = ( (1-t) * sy ) + ( t * dy );

		t += toffset;

		if ( graph[round(x)][round(y)].blocked == true )
			return 0;
	}

	return 1;
}